@@ -11,13 +11,11 @@ package checker
 import (
 	"bytes"
 	"encoding/gob"
-	"errors"
 	"flag"
 	"fmt"
 	"go/format"
 	"go/token"
 	"go/types"
-	"io/ioutil"
 	"log"
 	"os"
 	"reflect"
@@ -32,6 +30,7 @@ import (
 	"golang.org/x/tools/go/analysis"
 	"golang.org/x/tools/go/analysis/internal/analysisflags"
 	"golang.org/x/tools/go/packages"
+	"golang.org/x/tools/internal/analysisinternal"
 	"golang.org/x/tools/internal/diff"
 	"golang.org/x/tools/internal/robustio"
 )
@@ -134,16 +133,21 @@ func Run(args []string, analyzers []*analysis.Analyzer) (exitcode int) {
 	allSyntax := needFacts(analyzers)
 	initial, err := load(args, allSyntax)
 	if err != nil {
-		if _, ok := err.(typeParseError); !ok {
-			// Fail when some of the errors are not
-			// related to parsing nor typing.
-			log.Print(err)
-			return 1
-		}
-		// TODO: filter analyzers based on RunDespiteError?
+		log.Print(err)
+		return 1
 	}
 
-	// Run the analysis.
+	pkgsExitCode := 0
+	// Print package and module errors regardless of RunDespiteErrors.
+	// Do not exit if there are errors, yet.
+	if n := packages.PrintErrors(initial); n > 0 {
+		pkgsExitCode = 1
+	}
+
+	// Run the analyzers. On each package with (transitive)
+	// errors, we run only the subset of analyzers that are
+	// marked (and whose transitive requirements are also
+	// marked) with RunDespiteErrors.
 	roots := analyze(initial, analyzers)
 
 	// Apply fixes.
@@ -155,18 +159,18 @@ func Run(args []string, analyzers []*analysis.Analyzer) (exitcode int) {
 		}
 	}
 
-	// Print the results.
-	return printDiagnostics(roots)
+	// Print the results. If !RunDespiteErrors and there
+	// are errors in the packages, this will have 0 exit
+	// code. Otherwise, we prefer to return exit code
+	// indicating diagnostics.
+	if diagExitCode := printDiagnostics(roots); diagExitCode != 0 {
+		return diagExitCode // there were diagnostics
+	}
+	return pkgsExitCode // package errors but no diagnostics
 }
 
-// typeParseError represents a package load error
-// that is related to typing and parsing.
-type typeParseError struct {
-	error
-}
-
-// load loads the initial packages. If all loading issues are related to
-// typing and parsing, the returned error is of type typeParseError.
+// load loads the initial packages. Returns only top-level loading
+// errors. Does not consider errors in packages.
 func load(patterns []string, allSyntax bool) ([]*packages.Package, error) {
 	mode := packages.LoadSyntax
 	if allSyntax {
@@ -178,44 +182,12 @@ func load(patterns []string, allSyntax bool) ([]*packages.Package, error) {
 		Tests: IncludeTests,
 	}
 	initial, err := packages.Load(&conf, patterns...)
-	if err == nil {
-		if len(initial) == 0 {
-			err = fmt.Errorf("%s matched no packages", strings.Join(patterns, " "))
-		} else {
-			err = loadingError(initial)
-		}
+	if err == nil && len(initial) == 0 {
+		err = fmt.Errorf("%s matched no packages", strings.Join(patterns, " "))
 	}
 	return initial, err
 }
 
-// loadingError checks for issues during the loading of initial
-// packages. Returns nil if there are no issues. Returns error
-// of type typeParseError if all errors, including those in
-// dependencies, are related to typing or parsing. Otherwise,
-// a plain error is returned with an appropriate message.
-func loadingError(initial []*packages.Package) error {
-	var err error
-	if n := packages.PrintErrors(initial); n > 1 {
-		err = fmt.Errorf("%d errors during loading", n)
-	} else if n == 1 {
-		err = errors.New("error during loading")
-	} else {
-		// no errors
-		return nil
-	}
-	all := true
-	packages.Visit(initial, nil, func(pkg *packages.Package) {
-		for _, err := range pkg.Errors {
-			typeOrParse := err.Kind == packages.TypeError || err.Kind == packages.ParseError
-			all = all && typeOrParse
-		}
-	})
-	if all {
-		return typeParseError{err}
-	}
-	return err
-}
-
 // TestAnalyzer applies an analyzer to a set of packages (and their
 // dependencies if necessary) and returns the results.
 // The analyzer must be valid according to [analysis.Validate].
@@ -343,20 +315,28 @@ func applyFixes(roots []*action) error {
 				for _, edit := range sf.TextEdits {
 					// Validate the edit.
 					// Any error here indicates a bug in the analyzer.
-					file := act.pkg.Fset.File(edit.Pos)
+					start, end := edit.Pos, edit.End
+					file := act.pkg.Fset.File(start)
 					if file == nil {
 						return fmt.Errorf("analysis %q suggests invalid fix: missing file info for pos (%v)",
-							act.a.Name, edit.Pos)
+							act.a.Name, start)
 					}
-					if edit.Pos > edit.End {
+					if !end.IsValid() {
+						end = start
+					}
+					if start > end {
 						return fmt.Errorf("analysis %q suggests invalid fix: pos (%v) > end (%v)",
-							act.a.Name, edit.Pos, edit.End)
+							act.a.Name, start, end)
 					}
-					if eof := token.Pos(file.Base() + file.Size()); edit.End > eof {
+					if eof := token.Pos(file.Base() + file.Size()); end > eof {
 						return fmt.Errorf("analysis %q suggests invalid fix: end (%v) past end of file (%v)",
-							act.a.Name, edit.End, eof)
+							act.a.Name, end, eof)
+					}
+					edit := diff.Edit{
+						Start: file.Offset(start),
+						End:   file.Offset(end),
+						New:   string(edit.NewText),
 					}
-					edit := diff.Edit{Start: file.Offset(edit.Pos), End: file.Offset(edit.End), New: string(edit.NewText)}
 					editsForTokenFile[file] = append(editsForTokenFile[file], edit)
 				}
 			}
@@ -425,7 +405,9 @@ func applyFixes(roots []*action) error {
 
 	// Now we've got a set of valid edits for each file. Apply them.
 	for path, edits := range editsByPath {
-		contents, err := ioutil.ReadFile(path)
+		// TODO(adonovan): this should really work on the same
+		// gulp from the file system that fed the analyzer (see #62292).
+		contents, err := os.ReadFile(path)
 		if err != nil {
 			return err
 		}
@@ -440,7 +422,7 @@ func applyFixes(roots []*action) error {
 			out = formatted
 		}
 
-		if err := ioutil.WriteFile(path, out, 0644); err != nil {
+		if err := os.WriteFile(path, out, 0644); err != nil {
 			return err
 		}
 	}
@@ -480,17 +462,17 @@ func validateEdits(edits []diff.Edit) ([]diff.Edit, int) {
 // diff3Conflict returns an error describing two conflicting sets of
 // edits on a file at path.
 func diff3Conflict(path string, xlabel, ylabel string, xedits, yedits []diff.Edit) error {
-	contents, err := ioutil.ReadFile(path)
+	contents, err := os.ReadFile(path)
 	if err != nil {
 		return err
 	}
 	oldlabel, old := "base", string(contents)
 
-	xdiff, err := diff.ToUnified(oldlabel, xlabel, old, xedits)
+	xdiff, err := diff.ToUnified(oldlabel, xlabel, old, xedits, diff.DefaultContextLines)
 	if err != nil {
 		return err
 	}
-	ydiff, err := diff.ToUnified(oldlabel, ylabel, old, yedits)
+	ydiff, err := diff.ToUnified(oldlabel, ylabel, old, yedits, diff.DefaultContextLines)
 	if err != nil {
 		return err
 	}
@@ -738,6 +720,13 @@ func (act *action) execOnce() {
 		}
 	}
 
+	module := &analysis.Module{} // possibly empty (non nil) in go/analysis drivers.
+	if mod := act.pkg.Module; mod != nil {
+		module.Path = mod.Path
+		module.Version = mod.Version
+		module.GoVersion = mod.GoVersion
+	}
+
 	// Run the analysis.
 	pass := &analysis.Pass{
 		Analyzer:     act.a,
@@ -749,6 +738,7 @@ func (act *action) execOnce() {
 		TypesInfo:    act.pkg.TypesInfo,
 		TypesSizes:   act.pkg.TypesSizes,
 		TypeErrors:   act.pkg.TypeErrors,
+		Module:       module,
 
 		ResultOf:          inputs,
 		Report:            func(d analysis.Diagnostic) { act.diagnostics = append(act.diagnostics, d) },
@@ -759,6 +749,7 @@ func (act *action) execOnce() {
 		AllObjectFacts:    act.allObjectFacts,
 		AllPackageFacts:   act.allPackageFacts,
 	}
+	pass.ReadFile = analysisinternal.MakeReadFile(pass)
 	act.pass = pass
 
 	var err error
