@@ -9,9 +9,10 @@ import (
 	"go/token"
 	"go/types"
 
-	"golang.org/x/tools/go/callgraph"
 	"golang.org/x/tools/go/ssa"
 	"golang.org/x/tools/go/types/typeutil"
+	"golang.org/x/tools/internal/aliases"
+	"golang.org/x/tools/internal/typeparams"
 )
 
 // node interface for VTA nodes.
@@ -105,12 +106,12 @@ type field struct {
 }
 
 func (f field) Type() types.Type {
-	s := f.StructType.Underlying().(*types.Struct)
+	s := typeparams.CoreType(f.StructType).(*types.Struct)
 	return s.Field(f.index).Type()
 }
 
 func (f field) String() string {
-	s := f.StructType.Underlying().(*types.Struct)
+	s := typeparams.CoreType(f.StructType).(*types.Struct)
 	return fmt.Sprintf("Field(%v:%s)", f.StructType, s.Field(f.index).Name())
 }
 
@@ -170,6 +171,26 @@ func (f function) String() string {
 	return fmt.Sprintf("Function(%s)", f.f.Name())
 }
 
+// resultVar represents the result
+// variable of a function, whether
+// named or not.
+type resultVar struct {
+	f     *ssa.Function
+	index int // valid index into result var tuple
+}
+
+func (o resultVar) Type() types.Type {
+	return o.f.Signature.Results().At(o.index).Type()
+}
+
+func (o resultVar) String() string {
+	v := o.f.Signature.Results().At(o.index)
+	if n := v.Name(); n != "" {
+		return fmt.Sprintf("Return(%s[%s])", o.f.Name(), n)
+	}
+	return fmt.Sprintf("Return(%s[%d])", o.f.Name(), o.index)
+}
+
 // nestedPtrInterface node represents all references and dereferences
 // of locals and globals that have a nested pointer to interface type.
 // We merge such constructs into a single node for simplicity and without
@@ -233,35 +254,27 @@ func (r recoverReturn) String() string {
 	return "Recover"
 }
 
+type empty = struct{}
+
 // vtaGraph remembers for each VTA node the set of its successors.
 // Tailored for VTA, hence does not support singleton (sub)graphs.
-type vtaGraph map[node]map[node]bool
+type vtaGraph map[node]map[node]empty
 
 // addEdge adds an edge x->y to the graph.
 func (g vtaGraph) addEdge(x, y node) {
 	succs, ok := g[x]
 	if !ok {
-		succs = make(map[node]bool)
+		succs = make(map[node]empty)
 		g[x] = succs
 	}
-	succs[y] = true
-}
-
-// successors returns all of n's immediate successors in the graph.
-// The order of successor nodes is arbitrary.
-func (g vtaGraph) successors(n node) []node {
-	var succs []node
-	for succ := range g[n] {
-		succs = append(succs, succ)
-	}
-	return succs
+	succs[y] = empty{}
 }
 
 // typePropGraph builds a VTA graph for a set of `funcs` and initial
 // `callgraph` needed to establish interprocedural edges. Returns the
 // graph and a map for unique type representatives.
-func typePropGraph(funcs map[*ssa.Function]bool, callgraph *callgraph.Graph) (vtaGraph, *typeutil.Map) {
-	b := builder{graph: make(vtaGraph), callGraph: callgraph}
+func typePropGraph(funcs map[*ssa.Function]bool, callees calleesFunc) (vtaGraph, *typeutil.Map) {
+	b := builder{graph: make(vtaGraph), callees: callees}
 	b.visit(funcs)
 	return b.graph, &b.canon
 }
@@ -269,8 +282,8 @@ func typePropGraph(funcs map[*ssa.Function]bool, callgraph *callgraph.Graph) (vt
 // Data structure responsible for linearly traversing the
 // code and building a VTA graph.
 type builder struct {
-	graph     vtaGraph
-	callGraph *callgraph.Graph // initial call graph for creating flows at unresolved call sites.
+	graph   vtaGraph
+	callees calleesFunc // initial call graph for creating flows at unresolved call sites.
 
 	// Specialized type map for canonicalization of types.Type.
 	// Semantically equivalent types can have different implementations,
@@ -327,14 +340,16 @@ func (b *builder) instr(instr ssa.Instruction) {
 		// change type command a := A(b) results in a and b being the
 		// same value. For concrete type A, there is no interesting flow.
 		//
-		// Note: When A is an interface, most interface casts are handled
+		// When A is an interface, most interface casts are handled
 		// by the ChangeInterface instruction. The relevant case here is
 		// when converting a pointer to an interface type. This can happen
 		// when the underlying interfaces have the same method set.
-		//   type I interface{ foo() }
-		//   type J interface{ foo() }
-		//   var b *I
-		//   a := (*J)(b)
+		//
+		//	type I interface{ foo() }
+		//	type J interface{ foo() }
+		//	var b *I
+		//	a := (*J)(b)
+		//
 		// When this happens we add flows between a <--> b.
 		b.addInFlowAliasEdges(b.nodeFromVal(i), b.nodeFromVal(i.X))
 	case *ssa.TypeAssert:
@@ -373,6 +388,8 @@ func (b *builder) instr(instr ssa.Instruction) {
 		// SliceToArrayPointer: t1 = slice to array pointer *[4]T <- []T (t0)
 		// No interesting flow as sliceArrayElem(t1) == sliceArrayElem(t0).
 		return
+	case *ssa.MultiConvert:
+		b.multiconvert(i)
 	default:
 		panic(fmt.Sprintf("unsupported instruction %v\n", instr))
 	}
@@ -384,7 +401,7 @@ func (b *builder) unop(u *ssa.UnOp) {
 		// Multiplication operator * is used here as a dereference operator.
 		b.addInFlowAliasEdges(b.nodeFromVal(u), b.nodeFromVal(u.X))
 	case token.ARROW:
-		t := u.X.Type().Underlying().(*types.Chan).Elem()
+		t := typeparams.CoreType(u.X.Type()).(*types.Chan).Elem()
 		b.addInFlowAliasEdges(b.nodeFromVal(u), channelElem{typ: t})
 	default:
 		// There is no interesting type flow otherwise.
@@ -405,7 +422,7 @@ func (b *builder) tassert(a *ssa.TypeAssert) {
 	// The case where a is <a.AssertedType, bool> register so there
 	// is a flow from a.X to a[0]. Here, a[0] is represented as an
 	// indexedLocal: an entry into local tuple register a at index 0.
-	tup := a.Type().Underlying().(*types.Tuple)
+	tup := a.Type().(*types.Tuple)
 	t := tup.At(0).Type()
 
 	local := indexedLocal{val: a, typ: t, index: 0}
@@ -416,7 +433,7 @@ func (b *builder) tassert(a *ssa.TypeAssert) {
 // and t1 where the source is indexed local representing a value
 // from tuple register t2 at index i and the target is t1.
 func (b *builder) extract(e *ssa.Extract) {
-	tup := e.Tuple.Type().Underlying().(*types.Tuple)
+	tup := e.Tuple.Type().(*types.Tuple)
 	t := tup.At(e.Index).Type()
 
 	local := indexedLocal{val: e.Tuple, typ: t, index: e.Index}
@@ -429,7 +446,7 @@ func (b *builder) field(f *ssa.Field) {
 }
 
 func (b *builder) fieldAddr(f *ssa.FieldAddr) {
-	t := f.X.Type().Underlying().(*types.Pointer).Elem()
+	t := typeparams.CoreType(f.X.Type()).(*types.Pointer).Elem()
 
 	// Since we are getting pointer to a field, make a bidirectional edge.
 	fnode := field{StructType: t, index: f.Field}
@@ -438,7 +455,7 @@ func (b *builder) fieldAddr(f *ssa.FieldAddr) {
 }
 
 func (b *builder) send(s *ssa.Send) {
-	t := s.Chan.Type().Underlying().(*types.Chan).Elem()
+	t := typeparams.CoreType(s.Chan.Type()).(*types.Chan).Elem()
 	b.addInFlowAliasEdges(channelElem{typ: t}, b.nodeFromVal(s.X))
 }
 
@@ -452,7 +469,7 @@ func (b *builder) send(s *ssa.Send) {
 func (b *builder) selekt(s *ssa.Select) {
 	recvIndex := 0
 	for _, state := range s.States {
-		t := state.Chan.Type().Underlying().(*types.Chan).Elem()
+		t := typeparams.CoreType(state.Chan.Type()).(*types.Chan).Elem()
 
 		if state.Dir == types.SendOnly {
 			b.addInFlowAliasEdges(channelElem{typ: t}, b.nodeFromVal(state.Send))
@@ -492,7 +509,13 @@ func (b *builder) lookup(l *ssa.Lookup) {
 		// No interesting flows for string lookups.
 		return
 	}
-	b.addInFlowAliasEdges(b.nodeFromVal(l), mapValue{typ: t.Elem()})
+
+	if !l.CommaOk {
+		b.addInFlowAliasEdges(b.nodeFromVal(l), mapValue{typ: t.Elem()})
+	} else {
+		i := indexedLocal{val: l, typ: t.Elem(), index: 0}
+		b.addInFlowAliasEdges(i, mapValue{typ: t.Elem()})
+	}
 }
 
 // mapUpdate handles map update commands m[b] = a where m is of type
@@ -516,7 +539,7 @@ func (b *builder) next(n *ssa.Next) {
 	if n.IsString {
 		return
 	}
-	tup := n.Type().Underlying().(*types.Tuple)
+	tup := n.Type().(*types.Tuple)
 	kt := tup.At(1).Type()
 	vt := tup.At(2).Type()
 
@@ -568,13 +591,34 @@ func (b *builder) panic(p *ssa.Panic) {
 func (b *builder) call(c ssa.CallInstruction) {
 	// When c is r := recover() call register instruction, we add Recover -> r.
 	if bf, ok := c.Common().Value.(*ssa.Builtin); ok && bf.Name() == "recover" {
-		b.addInFlowEdge(recoverReturn{}, b.nodeFromVal(c.(*ssa.Call)))
+		if v, ok := c.(ssa.Value); ok {
+			b.addInFlowEdge(recoverReturn{}, b.nodeFromVal(v))
+		}
 		return
 	}
 
-	for _, f := range siteCallees(c, b.callGraph) {
+	siteCallees(c, b.callees)(func(f *ssa.Function) bool {
 		addArgumentFlows(b, c, f)
-	}
+
+		site, ok := c.(ssa.Value)
+		if !ok {
+			return true // go or defer
+		}
+
+		results := f.Signature.Results()
+		if results.Len() == 1 {
+			// When there is only one return value, the destination register does not
+			// have a tuple type.
+			b.addInFlowEdge(resultVar{f: f, index: 0}, b.nodeFromVal(site))
+		} else {
+			tup := site.Type().(*types.Tuple)
+			for i := 0; i < results.Len(); i++ {
+				local := indexedLocal{val: site, typ: tup.At(i).Type(), index: i}
+				b.addInFlowEdge(resultVar{f: f, index: i}, local)
+			}
+		}
+		return true
+	})
 }
 
 func addArgumentFlows(b *builder, c ssa.CallInstruction, f *ssa.Function) {
@@ -586,14 +630,22 @@ func addArgumentFlows(b *builder, c ssa.CallInstruction, f *ssa.Function) {
 		return
 	}
 	cc := c.Common()
+	if cc.Method != nil {
+		// In principle we don't add interprocedural flows for receiver
+		// objects. At a call site, the receiver object is interface
+		// while the callee object is concrete. The flow from interface
+		// to concrete type in general does not make sense. The exception
+		// is when the concrete type is a named function type (see #57756).
+		//
+		// The flow other way around would bake in information from the
+		// initial call graph.
+		if isFunction(f.Params[0].Type()) {
+			b.addInFlowEdge(b.nodeFromVal(cc.Value), b.nodeFromVal(f.Params[0]))
+		}
+	}
 
 	offset := 0
 	if cc.Method != nil {
-		// We don't add interprocedural flows for receiver objects.
-		// At a call site, the receiver object is interface while the
-		// callee object is concrete. The flow from interface to
-		// concrete type does not make sense. The flow other way around
-		// would bake in information from the initial call graph.
 		offset = 1
 	}
 	for i, v := range cc.Args {
@@ -609,37 +661,76 @@ func addArgumentFlows(b *builder, c ssa.CallInstruction, f *ssa.Function) {
 	}
 }
 
-// rtrn produces flows between values of r and c where
-// c is a call instruction that resolves to the enclosing
-// function of r based on b.callGraph.
+// rtrn creates flow edges from the operands of the return
+// statement to the result variables of the enclosing function.
 func (b *builder) rtrn(r *ssa.Return) {
-	n := b.callGraph.Nodes[r.Parent()]
-	// n != nil when b.callgraph is sound, but the client can
-	// pass any callgraph, including an underapproximate one.
-	if n == nil {
-		return
-	}
-
-	for _, e := range n.In {
-		if cv, ok := e.Site.(ssa.Value); ok {
-			addReturnFlows(b, r, cv)
-		}
+	for i, rs := range r.Results {
+		b.addInFlowEdge(b.nodeFromVal(rs), resultVar{f: r.Parent(), index: i})
 	}
 }
 
-func addReturnFlows(b *builder, r *ssa.Return, site ssa.Value) {
-	results := r.Results
-	if len(results) == 1 {
-		// When there is only one return value, the destination register does not
-		// have a tuple type.
-		b.addInFlowEdge(b.nodeFromVal(results[0]), b.nodeFromVal(site))
-		return
-	}
+func (b *builder) multiconvert(c *ssa.MultiConvert) {
+	// TODO(zpavlinovic): decide what to do on MultiConvert long term.
+	// TODO(zpavlinovic): add unit tests.
+	typeSetOf := func(typ types.Type) []*types.Term {
+		// This is a adaptation of x/exp/typeparams.NormalTerms which x/tools cannot depend on.
+		var terms []*types.Term
+		var err error
+		switch typ := aliases.Unalias(typ).(type) {
+		case *types.TypeParam:
+			terms, err = typeparams.StructuralTerms(typ)
+		case *types.Union:
+			terms, err = typeparams.UnionTermSet(typ)
+		case *types.Interface:
+			terms, err = typeparams.InterfaceTermSet(typ)
+		default:
+			// Common case.
+			// Specializing the len=1 case to avoid a slice
+			// had no measurable space/time benefit.
+			terms = []*types.Term{types.NewTerm(false, typ)}
+		}
 
-	tup := site.Type().Underlying().(*types.Tuple)
-	for i, r := range results {
-		local := indexedLocal{val: site, typ: tup.At(i).Type(), index: i}
-		b.addInFlowEdge(b.nodeFromVal(r), local)
+		if err != nil {
+			return nil
+		}
+		return terms
+	}
+	// isValuePreserving returns true if a conversion from ut_src to
+	// ut_dst is value-preserving, i.e. just a change of type.
+	// Precondition: neither argument is a named or alias type.
+	isValuePreserving := func(ut_src, ut_dst types.Type) bool {
+		// Identical underlying types?
+		if types.IdenticalIgnoreTags(ut_dst, ut_src) {
+			return true
+		}
+
+		switch ut_dst.(type) {
+		case *types.Chan:
+			// Conversion between channel types?
+			_, ok := ut_src.(*types.Chan)
+			return ok
+
+		case *types.Pointer:
+			// Conversion between pointers with identical base types?
+			_, ok := ut_src.(*types.Pointer)
+			return ok
+		}
+		return false
+	}
+	dst_terms := typeSetOf(c.Type())
+	src_terms := typeSetOf(c.X.Type())
+	for _, s := range src_terms {
+		us := s.Type().Underlying()
+		for _, d := range dst_terms {
+			ud := d.Type().Underlying()
+			if isValuePreserving(us, ud) {
+				// This is equivalent to a ChangeType.
+				b.addInFlowAliasEdges(b.nodeFromVal(c), b.nodeFromVal(c.X))
+				return
+			}
+			// This is equivalent to either: SliceToArrayPointer,,
+			// SliceToArrayPointer+Deref, Size 0 Array constant, or a Convert.
+		}
 	}
 }
 
@@ -654,7 +745,7 @@ func (b *builder) addInFlowEdge(s, d node) {
 
 // Creates const, pointer, global, func, and local nodes based on register instructions.
 func (b *builder) nodeFromVal(val ssa.Value) node {
-	if p, ok := val.Type().(*types.Pointer); ok && !types.IsInterface(p.Elem()) && !isFunction(p.Elem()) {
+	if p, ok := aliases.Unalias(val.Type()).(*types.Pointer); ok && !types.IsInterface(p.Elem()) && !isFunction(p.Elem()) {
 		// Nested pointer to interfaces are modeled as a special
 		// nestedPtrInterface node.
 		if i := interfaceUnderPtr(p.Elem()); i != nil {
@@ -715,7 +806,7 @@ func (b *builder) representative(n node) node {
 		return field{StructType: canonicalize(i.StructType, &b.canon), index: i.index}
 	case indexedLocal:
 		return indexedLocal{typ: t, val: i.val, index: i.index}
-	case local, global, panicArg, recoverReturn, function:
+	case local, global, panicArg, recoverReturn, function, resultVar:
 		return n
 	default:
 		panic(fmt.Errorf("canonicalizing unrecognized node %v", n))
