@@ -8,7 +8,6 @@ import (
 	"archive/zip"
 	"context"
 	"fmt"
-	"io/ioutil"
 	"log"
 	"os"
 	"path/filepath"
@@ -18,6 +17,7 @@ import (
 	"strings"
 	"sync"
 	"testing"
+	"time"
 
 	"golang.org/x/mod/module"
 	"golang.org/x/tools/internal/gocommand"
@@ -94,7 +94,7 @@ package z
 
 	mt.assertFound("y", "y")
 
-	scan, err := scanToSlice(mt.resolver, nil)
+	scan, err := scanToSlice(mt.env.resolver, nil)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -197,7 +197,7 @@ import _ "rsc.io/quote"
 	if err := os.Chmod(filepath.Join(found.dir, "go.mod"), 0644); err != nil {
 		t.Fatal(err)
 	}
-	if err := ioutil.WriteFile(filepath.Join(found.dir, "go.mod"), []byte("module bad.com\n"), 0644); err != nil {
+	if err := os.WriteFile(filepath.Join(found.dir, "go.mod"), []byte("module bad.com\n"), 0644); err != nil {
 		t.Fatal(err)
 	}
 
@@ -205,15 +205,15 @@ import _ "rsc.io/quote"
 	mt.assertScanFinds("rsc.io/quote", "quote")
 
 	// Rewrite the main package so that rsc.io/quote is not in scope.
-	if err := ioutil.WriteFile(filepath.Join(mt.env.WorkingDir, "go.mod"), []byte("module x\n"), 0644); err != nil {
+	if err := os.WriteFile(filepath.Join(mt.env.WorkingDir, "go.mod"), []byte("module x\n"), 0644); err != nil {
 		t.Fatal(err)
 	}
-	if err := ioutil.WriteFile(filepath.Join(mt.env.WorkingDir, "x.go"), []byte("package x\n"), 0644); err != nil {
+	if err := os.WriteFile(filepath.Join(mt.env.WorkingDir, "x.go"), []byte("package x\n"), 0644); err != nil {
 		t.Fatal(err)
 	}
 
 	// Uninitialize the go.mod dependent cached information and make sure it still finds the package.
-	mt.resolver.ClearForNewMod()
+	mt.env.ClearModuleInfo()
 	mt.assertScanFinds("rsc.io/quote", "quote")
 }
 
@@ -242,8 +242,9 @@ import _ "rsc.io/sampler"
 	}
 
 	// Clear out the resolver's cache, since we've changed the environment.
-	mt.resolver = newModuleResolver(mt.env)
 	mt.env.Env["GOFLAGS"] = "-mod=vendor"
+	mt.env.ClearModuleInfo()
+	mt.env.UpdateResolver(mt.env.resolver.ClearForNewScan())
 	mt.assertModuleFoundInDir("rsc.io/sampler", "sampler", `/vendor/`)
 }
 
@@ -270,6 +271,10 @@ import _ "rsc.io/sampler"
 	if testenv.Go1Point() >= 14 {
 		wantDir = `/vendor/`
 	}
+
+	// Clear out the resolver's module info, since we've changed the environment.
+	// (the presence of a /vendor directory affects `go list -m`).
+	mt.env.ClearModuleInfo()
 	mt.assertModuleFoundInDir("rsc.io/sampler", "sampler", wantDir)
 }
 
@@ -554,8 +559,6 @@ package v
 
 // Tests that go.work files are respected.
 func TestModWorkspace(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18)
-
 	mt := setup(t, nil, `
 -- go.work --
 go 1.18
@@ -590,8 +593,6 @@ package b
 // respected and that a wildcard replace in go.work overrides a versioned replace
 // in go.mod.
 func TestModWorkspaceReplace(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18)
-
 	mt := setup(t, nil, `
 -- go.work --
 use m
@@ -649,8 +650,6 @@ func G() {
 // Tests a case where conflicting replaces are overridden by a replace
 // in the go.work file.
 func TestModWorkspaceReplaceOverride(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18)
-
 	mt := setup(t, nil, `-- go.work --
 use m
 use n
@@ -714,8 +713,6 @@ func G() {
 // workspaces with module pruning. This is based on the
 // cmd/go mod_prune_all script test.
 func TestModWorkspacePrune(t *testing.T) {
-	testenv.NeedsGo1Point(t, 18)
-
 	mt := setup(t, nil, `
 -- go.work --
 go 1.18
@@ -901,7 +898,7 @@ package x
 func (t *modTest) assertFound(importPath, pkgName string) (string, *pkg) {
 	t.Helper()
 
-	names, err := t.resolver.loadPackageNames([]string{importPath}, t.env.WorkingDir)
+	names, err := t.env.resolver.loadPackageNames([]string{importPath}, t.env.WorkingDir)
 	if err != nil {
 		t.Errorf("loading package name for %v: %v", importPath, err)
 	}
@@ -910,13 +907,13 @@ func (t *modTest) assertFound(importPath, pkgName string) (string, *pkg) {
 	}
 	pkg := t.assertScanFinds(importPath, pkgName)
 
-	_, foundDir := t.resolver.findPackage(importPath)
+	_, foundDir := t.env.resolver.(*ModuleResolver).findPackage(importPath)
 	return foundDir, pkg
 }
 
 func (t *modTest) assertScanFinds(importPath, pkgName string) *pkg {
 	t.Helper()
-	scan, err := scanToSlice(t.resolver, nil)
+	scan, err := scanToSlice(t.env.resolver, nil)
 	if err != nil {
 		t.Errorf("scan failed: %v", err)
 	}
@@ -984,10 +981,9 @@ var proxyDir string
 
 type modTest struct {
 	*testing.T
-	env      *ProcessEnv
-	gopath   string
-	resolver *ModuleResolver
-	cleanup  func()
+	env     *ProcessEnv
+	gopath  string
+	cleanup func()
 }
 
 // setup builds a test environment from a txtar and supporting modules
@@ -1000,7 +996,7 @@ func setup(t *testing.T, extraEnv map[string]string, main, wd string) *modTest {
 
 	proxyOnce.Do(func() {
 		var err error
-		proxyDir, err = ioutil.TempDir("", "proxy-")
+		proxyDir, err = os.MkdirTemp("", "proxy-")
 		if err != nil {
 			t.Fatal(err)
 		}
@@ -1009,7 +1005,7 @@ func setup(t *testing.T, extraEnv map[string]string, main, wd string) *modTest {
 		}
 	})
 
-	dir, err := ioutil.TempDir("", t.Name())
+	dir, err := os.MkdirTemp("", t.Name())
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -1047,16 +1043,20 @@ func setup(t *testing.T, extraEnv map[string]string, main, wd string) *modTest {
 		}
 	}
 
-	resolver, err := env.GetResolver()
-	if err != nil {
+	// Ensure the resolver is set for tests that (unsafely) access env.resolver
+	// directly.
+	//
+	// TODO(rfindley): fix this after addressing the TODO in the ProcessEnv
+	// docstring.
+	if _, err := env.GetResolver(); err != nil {
 		t.Fatal(err)
 	}
+
 	return &modTest{
-		T:        t,
-		gopath:   env.Env["GOPATH"],
-		env:      env,
-		resolver: resolver.(*ModuleResolver),
-		cleanup:  func() { removeDir(dir) },
+		T:       t,
+		gopath:  env.Env["GOPATH"],
+		env:     env,
+		cleanup: func() { removeDir(dir) },
 	}
 }
 
@@ -1070,7 +1070,7 @@ func writeModule(dir, ar string) error {
 			return err
 		}
 
-		if err := ioutil.WriteFile(fpath, f.Data, 0644); err != nil {
+		if err := os.WriteFile(fpath, f.Data, 0644); err != nil {
 			return err
 		}
 	}
@@ -1080,7 +1080,7 @@ func writeModule(dir, ar string) error {
 // writeProxy writes all the txtar-formatted modules in arDir to a proxy
 // directory in dir.
 func writeProxy(dir, arDir string) error {
-	files, err := ioutil.ReadDir(arDir)
+	files, err := os.ReadDir(arDir)
 	if err != nil {
 		return err
 	}
@@ -1099,7 +1099,7 @@ func writeProxyModule(base, arPath string) error {
 	arName := filepath.Base(arPath)
 	i := strings.LastIndex(arName, "_v")
 	ver := strings.TrimSuffix(arName[i+1:], ".txt")
-	modDir := strings.Replace(arName[:i], "_", "/", -1)
+	modDir := strings.ReplaceAll(arName[:i], "_", "/")
 	modPath, err := module.UnescapePath(modDir)
 	if err != nil {
 		return err
@@ -1123,7 +1123,7 @@ func writeProxyModule(base, arPath string) error {
 	z := zip.NewWriter(f)
 	for _, f := range a.Files {
 		if f.Name[0] == '.' {
-			if err := ioutil.WriteFile(filepath.Join(dir, ver+f.Name), f.Data, 0644); err != nil {
+			if err := os.WriteFile(filepath.Join(dir, ver+f.Name), f.Data, 0644); err != nil {
 				return err
 			}
 		} else {
@@ -1184,7 +1184,7 @@ import _ "rsc.io/quote"
 	want := filepath.Join(mt.gopath, "pkg/mod", "rsc.io/quote@v1.5.2")
 
 	found := mt.assertScanFinds("rsc.io/quote", "quote")
-	modDir, _ := mt.resolver.modInfo(found.dir)
+	modDir, _ := mt.env.resolver.(*ModuleResolver).modInfo(found.dir)
 	if modDir != want {
 		t.Errorf("expected: %s, got: %s", want, modDir)
 	}
@@ -1194,7 +1194,7 @@ import _ "rsc.io/quote"
 func TestInvalidModCache(t *testing.T) {
 	testenv.NeedsTool(t, "go")
 
-	dir, err := ioutil.TempDir("", t.Name())
+	dir, err := os.MkdirTemp("", t.Name())
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -1204,7 +1204,7 @@ func TestInvalidModCache(t *testing.T) {
 	if err := os.MkdirAll(filepath.Join(dir, "gopath/pkg/mod/sabotage"), 0777); err != nil {
 		t.Fatal(err)
 	}
-	if err := ioutil.WriteFile(filepath.Join(dir, "gopath/pkg/mod/sabotage/x.go"), []byte("package foo\n"), 0777); err != nil {
+	if err := os.WriteFile(filepath.Join(dir, "gopath/pkg/mod/sabotage/x.go"), []byte("package foo\n"), 0777); err != nil {
 		t.Fatal(err)
 	}
 	env := &ProcessEnv{
@@ -1289,20 +1289,37 @@ import (
 	}
 }
 
-func BenchmarkScanModCache(b *testing.B) {
+func BenchmarkModuleResolver_RescanModCache(b *testing.B) {
 	env := &ProcessEnv{
 		GocmdRunner: &gocommand.Runner{},
-		Logf:        b.Logf,
+		// Uncomment for verbose logging (too verbose to enable by default).
+		// Logf:        b.Logf,
 	}
 	exclude := []gopathwalk.RootType{gopathwalk.RootGOROOT}
 	resolver, err := env.GetResolver()
 	if err != nil {
 		b.Fatal(err)
 	}
+	start := time.Now()
 	scanToSlice(resolver, exclude)
+	b.Logf("warming the mod cache took %v", time.Since(start))
 	b.ResetTimer()
 	for i := 0; i < b.N; i++ {
 		scanToSlice(resolver, exclude)
-		resolver.(*ModuleResolver).ClearForNewScan()
+		resolver = resolver.ClearForNewScan()
+	}
+}
+
+func BenchmarkModuleResolver_InitialScan(b *testing.B) {
+	for i := 0; i < b.N; i++ {
+		env := &ProcessEnv{
+			GocmdRunner: &gocommand.Runner{},
+		}
+		exclude := []gopathwalk.RootType{gopathwalk.RootGOROOT}
+		resolver, err := env.GetResolver()
+		if err != nil {
+			b.Fatal(err)
+		}
+		scanToSlice(resolver, exclude)
 	}
 }
